/******************************************************************************
 * program:     rasimg library                                                *
 * function:    Executable for image comparison.                              *
 * modul:       gcomp.cc                                                      *
 * licency:     GPL or LGPL                                                   *
 ******************************************************************************/
#include <stddef.h>
#include <stdio.h>
//#include <io.h>

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "types.h"
#include "raster.h"

#ifndef __UNIX__
 #include <io.h>
#endif

#include <matrix.h>


//#include <alloc.h>

extern Image LoadPicture(const char *Name);
extern int SavePicture(const char *Name,const Image &Img);


static char Verbosity = 0;


const char *GetExtension(const char *FullName)
{
const char *NameStep;
int LastPos;

if(FullName==NULL) return(NULL);

LastPos=strlen(FullName);
NameStep=FullName+LastPos+1;
while(NameStep>FullName)
	{
	NameStep--;

	if(*NameStep=='/' || *NameStep=='\\' || *NameStep==':')
		break;
	if(*NameStep=='.')
		return(NameStep);
	}

return(FullName+LastPos);
}


/////////////////////Main CODE starts here/////////////////////////////////

/** Compare two Images. */
long Compare(Image & Img1, Image & Img2)
{
WORD y,x;
long Difference;
Raster2DAbstract *Ras1;
Raster2DAbstract *Ras2;
Raster1DAbstract *Ras_1;
Raster1DAbstract *Ras_2;

 Difference=0;

 Ras1 = Img1.Raster;
 Ras2 = Img2.Raster;

 if(Ras1==NULL || Ras2==NULL) return -1;
 if(Ras1->Data2D==NULL || Ras2->Data2D==NULL) return -2;
 if(Ras1->GetSize1D()!=Ras2->GetSize1D() || Ras1->Size2D!=Ras2->Size2D)
   {
   if(Verbosity>=0)
     {
     printf("  Wrong image size %dx%d compared to %dx%d ",
		Ras1->GetSize1D(),Ras1->Size2D, Ras2->GetSize1D(),Ras2->Size2D);
     }
   return -3;
   }

/* if AlineProc<>nil then
	AlineProc^.InitPassing(Obr1.y,'Comparing 2 images'); */

 if(Verbosity>=0)
   printf("  Image types I1:%X (pl%d); I2:%X (pl%d) ",Img1.ImageType(),Ras1->GetPlanes(),
			Img2.ImageType(),Ras2->GetPlanes());

 if(Img1.ImageType()==ImageTrueColor || Img2.ImageType()==ImageTrueColor ||
    (Img1.ImageType()==ImagePalette && Img2.ImageType()==ImagePalette) )
   {		// True color process is required
   for(y=0; y<Ras1->Size2D; y++)
    {
    for(x=0; x<Ras1->GetSize1D(); x++)
      {
      if(Img1.GetPixelRGB(x,y) != Img2.GetPixelRGB(x,y))
	{
	//DWORD d = Img1.GetPixelRGB(x,y);
        //DWORD e = Img2.GetPixelRGB(x,y);
        
	if(Difference<10)
	  printf("\n [%d %d]: %lX %lX ",x,y,Img1.GetPixelRGB(x,y),Img2.GetPixelRGB(x,y));
	if(Difference==10)
          printf("\nAnd more ...");
        Difference++;
        }
      }  
    }
   }
 else if(Ras1->GetPlanes()<= 32 && Ras2->GetPlanes()<=32)
   {
   for(y=0; y<Ras1->Size2D; y++)
    {
    Ras_1 = Ras1->GetRowRaster(y);
    Ras_2 = Ras2->GetRowRaster(y);
    if(Ras_1==NULL || Ras_2==NULL) 
      {
      printf("\nEmpty row for line %d",y);
      Difference++;
      continue;
      }

    for(x=0; x<Ras1->GetSize1D(); x++)
      {
      if(Ras_1->GetValue1D(x) != Ras_2->GetValue1D(x))
        {
	if(Difference<10)
          printf("\n [%d %d]: %X %X ",x,y,Img1.GetPixel(x,y),Img2.GetPixel(x,y));
	if(Difference==10)
          printf("\nAnd more ...");
	Difference++;
        }
      }  
    }
  }
 else
   {
   for(y=0; y<Ras1->Size2D; y++)
    for(x=0; x<Ras1->GetSize1D(); x++)
      {
      if(Ras1->GetValue2Dd(x,y) != Ras2->GetValue2Dd(x,y))
        {
	if(Difference<10)
	  printf(" [%d %d]: %f %f ",x,y,Ras1->GetValue2Dd(x,y),Ras2->GetValue2Dd(x,y));
	if(Difference==10)
          printf("\nAnd more ...");
	Difference++;
        }
      }  
  }

return Difference;
}


void PrintHeader(void)
{
char HdrPrinted=0;
  if(Verbosity>=0 && HdrPrinted==0)
    {

    puts("\n\r <<< GComp >>> Compare two graphical images (c)1997-2016 F&TSoft");
    HdrPrinted=1;
    }
}


int main(int argc,char *argv[])
{
Image I1,I2;
const char *Img1=NULL, *Img2=NULL;
const char *s;
TImageFileHandler *LoadPicture_XXX;
int i;
int Error = 0;

/* Parse Arguments */
for(i=1;i<argc;i++)
  {
  s=argv[i];
  if(s==NULL) continue;
  if( *s==0 ) continue;

  if(*s=='-' || *s=='/')
    {
    if(strcmp(s+1,"help")==0 || strcmp(s+1,"-help")==0 || strcmp(s+1,"?")==0)
      {
        printf("\n");
        for(LoadPicture_XXX=TImageFileHandler::first();LoadPicture_XXX!=NULL;LoadPicture_XXX=LoadPicture_XXX->next())
	  {
	  printf("%s %s%s; ",LoadPicture_XXX->extension(),
			     LoadPicture_XXX->LoadPicture==NULL?"":"R",
			     LoadPicture_XXX->SavePicture==NULL?"":"W");
	  }
	printf("\n/s            - silent mode"
	       "\n/s            - verbose"
		//"\n/NoScale      - means truncating image data"
	       //"\n(not working yet) /Gray         - discards all color informations"
	       //"\n/SEPARATE[R|G|B] - remove one channel from true color images"
	       "\n");
	return(0);
      }

    if(strcmp(s+1,"s") == 0)
      {
      Verbosity = -1;
      continue;
      }
    if(strcmp(s+1,"v") == 0)
      {
      Verbosity = +1;
      continue;
      }

    }

  if(Img1==NULL)
     Img1=s;
  else
     if(Img2==NULL) Img2=s;
  }
/* End of reading arguments */

  PrintHeader();
  if(Verbosity>=0) printf("\rLoading");
  //MarkTime=GetInterval();

  I1=LoadPicture(Img1);  
  if(I1.Raster==NULL && I1.Comment==NULL)
	{
	printf("\n\rCannot load 1'st picture:\"%s\" Error %d",Img1,Error);
	Error = -11;
	goto End;
	}

  I2=LoadPicture(Img2);
  if(I2.Raster==NULL && I2.Comment==NULL)
	{
	printf("\n\rCannot load 2'nd picture:\"%s\" Error %d",Img2,Error);
	Error = -12;
	goto End;
	}

  if(Verbosity>=0) printf("\rComparing ");
  i = Compare(I1,I2);
  if(i==0)
    {
    if(Verbosity>=0)
      printf("\rBoth images have same contents.\n\r");
    }
  else
    {
    Error = 1;
    if(i<0) printf("\nError %d occured during compare!\n\r",i);
       else printf("\nFound %d mismatches!\n\r",i);
    }
  //MarkTime=GetInterval();

End:
  return(Error);
}

